"""Base class for anything that isn't a block or UI element."""

import vector

import pygame.sprite, pygame.image

import ConfigParser, os.path

class Entity(pygame.sprite.Sprite):
    """A base class for all my non-block/UI items onscreen"""
    def __init__(self, level, image, pos):
        pygame.sprite.Sprite.__init__(self)
        self.image = pygame.image.load(os.path.normcase(os.path.normpath(image)))
        self.rect = self.image.get_rect()
        self.rect.topleft = pos
        self.level = level
        self.termvel = 63
        self.vector = vector.Vec2d(0, 0)
        self.vectors = {}
        self.add_vector("grav", (0, self.level.grav))
        
    def add_vector(self, vid, vec):
        """add a vector to compute, with name vid"""
        if not isinstance(vec, vector.Vec2d):
            vec = vector.Vec2d(vec)
        self.vectors.update([(vid, vec)])
        
    def get_newpos(self):
        self.vector.x, self.vector.y = 0,0
        for vec in self.vectors.values():
            self.vector += vec
        newpos = self.rect.move(self.vector.x, self.vector.y)
        return newpos
        
    def collidecheck(self, newpos):
        top = self.level.get_at(newpos.centerx, newpos.top)
        bottom = self.level.get_at(newpos.centerx, newpos.bottom)
        left = self.level.get_at(newpos.left, newpos.centery)
        right = self.level.get_at(newpos.right, newpos.centery)
        
        if top and top.solid == "true":
            newpos.top = top.rect.bottom
        if bottom and bottom.solid == "true":
            newpos.bottom = bottom.rect.top
        if right and right.solid == "true":
            newpos.right = right.rect.left
        if left and left.solid == "true":
            newpos.left = left.rect.right
            
        return newpos
        
    def update(self):
        below = self.level.get_at(self.rect.centerx, self.rect.inflate(2, 2).bottom)
        if below and below.solid == "true":
            self.vectors["grav"] = vector.Vec2d(0, self.level.grav)
            
        elif not below or below.solid == "false":        
            if self.vectors["grav"].y < self.termvel:
                self.vectors["grav"].y += self.level.grav
            elif self.vectors["grav"].y >= self.termvel:
                self.vectors["grav"].y = self.termvel
        
        self.rect.center = self.collidecheck(self.get_newpos()).center 
